use crate::jira_config::JiraConfig;
use gray_matter::engine::TOML;
use gray_matter::Matter;
use reqwest::header::{HeaderMap, HeaderValue, CONTENT_TYPE};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::fs;
use std::io::Write;
use std::path::PathBuf;
use tempfile::NamedTempFile;

#[derive(Debug, Deserialize, Serialize)]
struct TicketMetadata {
    #[serde(skip_serializing_if = "Option::is_none")]
    status: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    project: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    assignee: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    parent: Option<String>,
    #[serde(flatten)]
    extra: HashMap<String, serde_json::Value>,
}

#[derive(Debug, Serialize)]
struct JiraIssueRequest {
    fields: JiraIssueFields,
    update: Option<JiraIssueUpdate>,
}

#[derive(Debug, Serialize, Deserialize)]
struct JiraIssueFields {
    project: Project,
    summary: String,
    description: String,
    issuetype: IssueType,
    #[serde(skip_serializing_if = "Option::is_none")]
    assignee: Option<Assignee>,
    #[serde(skip_serializing_if = "Option::is_none")]
    status: Option<Status>,
}

#[derive(Debug, Serialize)]
struct JiraIssueUpdate {
    #[serde(skip_serializing_if = "Option::is_none")]
    parent: Option<Vec<ParentUpdate>>,
}

#[derive(Debug, Serialize)]
struct ParentUpdate {
    add: Parent,
}

#[derive(Debug, Serialize)]
struct Parent {
    key: String,
}

#[derive(Debug, Serialize, Deserialize)]
struct Assignee {
    name: String,
}

#[derive(Debug, Serialize, Deserialize)]
struct Project {
    key: String,
}

#[derive(Debug, Serialize, Deserialize)]
struct IssueType {
    name: String,
}

#[derive(Deserialize)]
struct JiraResponse {
    key: String,
}

#[derive(Deserialize)]
struct JiraSearchResponse {
    issues: Vec<JiraIssue>,
    total: u32,
}

#[derive(Deserialize)]
struct JiraIssue {
    key: String,
    fields: JiraIssueResponseFields,
}

#[derive(Deserialize)]
struct JiraIssueResponseFields {
    summary: String,
    status: Status,
    created: String,
}

#[derive(Debug, Serialize, Deserialize)]
struct Status {
    name: String,
}

fn get_ticket_template(
    ticket_metadata: &TicketMetadata,
) -> Result<String, Box<dyn std::error::Error>> {
    let matter = toml::to_string(ticket_metadata)?;
    let has_project = ticket_metadata.project.is_some();

    let template = format!(
        r#"---
# The ticket can contain the following properties:
# - status
# - project (required)
# - assignee
# - parent
# - labels
# - priority

{}
{}
---
# Title

[description]
"#,
        if has_project { "" } else { r#"project = """# },
        matter,
    );

    Ok(template)
}

fn get_editor() -> String {
    std::env::var("EDITOR")
        .or_else(|_| std::env::var("VISUAL"))
        .unwrap_or_else(|_| "vi".to_string())
}

fn parse_markdown(
    content: &str,
) -> Result<(String, String, TicketMetadata), Box<dyn std::error::Error>> {
    let matter = Matter::<TOML>::new();
    let result = matter.parse_with_struct::<TicketMetadata>(content);

    let (metadata, content) = match result {
        Some(x) => (x.data, x.content),
        None => {
            eprintln!("unexpected markdown content found");
            std::process::exit(1);
        }
    };

    // let metadata = result.data
    //     .and_then(|d| toml::from_str(d).ok())
    //     .unwrap_or_else(|| TicketMetadata {
    //         status: None,
    //         project: None,
    //         assignee: None,
    //         parent: None,
    //         extra: HashMap::new(),
    //     });
    //
    // let content = result.content;
    let mut lines = content.lines();

    // Get first non-empty line as title
    let title = lines
        .by_ref()
        .find(|line| !line.trim().is_empty())
        .unwrap_or("")
        .trim_start_matches('#')
        .trim()
        .to_string();

    // Rest is description
    let description = lines.collect::<Vec<&str>>().join("\n");

    Ok((title, description, metadata))
}

fn create_temp_markdown(project_key: Option<String>) -> Result<String, Box<dyn std::error::Error>> {
    let mut temp_file = NamedTempFile::new()?;

    let metadata = TicketMetadata {
        status: None,
        project: project_key,
        assignee: None,
        parent: None,
        extra: Default::default(),
    };

    let template = get_ticket_template(&metadata)?;

    temp_file.write_all(template.as_bytes())?;
    temp_file.flush()?;

    let editor = get_editor();
    let status = std::process::Command::new(editor)
        .arg(temp_file.path())
        .status()?;

    if !status.success() {
        return Err("Editor exited with non-zero status".into());
    }

    // Read the modified content
    let content = fs::read_to_string(temp_file.path())?;

    // temp_file will be automatically deleted when it goes out of scope
    Ok(content)
}
async fn create_jira_issue(
    config: &JiraConfig,
    project_key: &str,
    title: &str,
    description: &str,
    metadata: &TicketMetadata,
) -> Result<JiraResponse, Box<dyn std::error::Error>> {
    let client = reqwest::Client::new();

    let mut headers = HeaderMap::new();
    headers.insert(CONTENT_TYPE, HeaderValue::from_static("application/json"));

    let mut fields = JiraIssueFields {
        project: Project {
            key: project_key.to_string(),
        },
        summary: title.to_string(),
        description: description.to_string(),
        issuetype: IssueType {
            name: "Task".to_string(),
        },
        assignee: None,
        status: None,
    };

    // Add assignee if specified
    if let Some(assignee) = &metadata.assignee {
        fields.assignee = Some(Assignee {
            name: assignee.clone(),
        });
    }

    let mut update = None;

    // Add parent if specified
    if let Some(parent_key) = &metadata.parent {
        update = Some(JiraIssueUpdate {
            parent: Some(vec![ParentUpdate {
                add: Parent {
                    key: parent_key.clone(),
                },
            }]),
        });
    }

    let issue = JiraIssueRequest { fields, update };

    // println!("{:#?}", issue);
    // Ok(JiraResponse {
    //     key: format!("{}-1234", project_key),
    // })

    let response = client
        .post(format!("{}/rest/api/2/issue", config.url))
        .basic_auth(&config.email, Some(&config.api_token))
        .headers(headers)
        .json(&issue)
        .send()
        .await?;
    
    if !response.status().is_success() {
        let error_text = response.text().await?;
        return Err(format!("Failed to create issue: {}", error_text).into());
    }
    
    let issue_response = response.json::<JiraResponse>().await?;
    
    // Update status if specified (requires a separate API call)
    if let Some(status) = &metadata.status {
        update_issue_status(config, &issue_response.key, status).await?;
    }
    
    Ok(issue_response)
}

async fn update_issue_status(
    config: &JiraConfig,
    issue_key: &str,
    status: &str,
) -> Result<(), Box<dyn std::error::Error>> {
    let client = reqwest::Client::new();

    let transitions = client
        .get(format!(
            "{}/rest/api/2/issue/{}/transitions",
            config.url, issue_key
        ))
        .basic_auth(&config.email, Some(&config.api_token))
        .send()
        .await?
        .json::<serde_json::Value>()
        .await?;

    // Find the transition ID for the desired status
    let transition_id = transitions["transitions"]
        .as_array()
        .and_then(|t| {
            t.iter().find(|t| {
                t["to"]["name"]
                    .as_str()
                    .map_or(false, |s| s.eq_ignore_ascii_case(status))
            })
        })
        .and_then(|t| t["id"].as_str())
        .ok_or_else(|| format!("No transition found for status: {}", status))?;

    // Perform the transition
    let transition_payload = serde_json::json!({
        "transition": { "id": transition_id }
    });

    let response = client
        .post(format!(
            "{}/rest/api/2/issue/{}/transitions",
            config.url, issue_key
        ))
        .basic_auth(&config.email, Some(&config.api_token))
        .json(&transition_payload)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(format!("Failed to update status: {}", response.text().await?).into());
    }

    Ok(())
}

pub async fn create(
    project: Option<String>,
    open: bool,
    markdown_file: Option<PathBuf>,
) -> Result<(), Box<dyn std::error::Error>> {
    let config = JiraConfig::load().map_err(|e| format!("Configuration error: {}", e))?;

    let content = match markdown_file {
        Some(path) => {
            fs::read_to_string(&path).map_err(|e| format!("Failed to read file: {}", e))?
        }
        None => {
            eprintln!("No markdown file specified. Opening editor...");
            create_temp_markdown(project)?
        }
    };

    if content.trim().is_empty() {
        return Err("Empty content. Aborting ticket creation.".into());
    }

    let (title, description, metadata) = parse_markdown(&content)?;

    let selected_project = match metadata.project.clone() {
        Some(x) => x.to_uppercase(),
        None => {
            eprintln!("No project set...");
            std::process::exit(1);
        }
    };

    if selected_project.is_empty() {
        eprintln!("No project set...");
        std::process::exit(1);
    }

    // Confirm creation
    println!("\nAbout to create ticket:");
    println!("Project: {}", selected_project);
    if let Some(status) = &metadata.status {
        println!("Status: {}", status);
    }
    if let Some(assignee) = &metadata.assignee {
        println!("Assignee: {}", assignee);
    }
    if let Some(parent) = &metadata.parent {
        println!("Parent: {}", parent);
    }
    println!("Title: {}", title);

    println!("Description:\n{}", description);
    println!("\nPress Enter to continue or Ctrl+C to abort");

    let mut input = String::new();
    std::io::stdin().read_line(&mut input)?;

    let response =
        create_jira_issue(&config, &selected_project, &title, &description, &metadata).await?;

    let url = format!("{}/browse/{}", config.url, response.key);

    println!("Successfully created ticket: {}", response.key);
    println!("URL: {}", url);

    if open {
        open::that(url)?;
    }

    Ok(())
}
